#include "lcdriver.h"

#include <stdio.h>
#include <string.h>
#include <time.h>


#define WIN32_LEAN_AND_MEAN

#include <windows.h>

#include "DLportio.h"

#ifdef _DEBUG
# define LOG( x ) if( log ) { x; }
#else
# define LOG( x )
#endif


int LPT_Data = 0x378;			// LPT1
int LPT_Ctrl = LPT_Data + 2;

int LongDelay = 4;
int ShortDelay = 2;

int cols = 20;
int rows = 4;

int cw = 5;
int ch = 7;

void LCD_Config( void )
{
	LONG err;
	HKEY key;
	err = RegOpenKey( HKEY_CURRENT_USER, "Software\\JSR\\LCD4", &key );
	if( err == NOERROR )
	{
		DWORD type = REG_DWORD;
		DWORD len = sizeof( DWORD );
		RegQueryValueEx( key, "LongDelay", NULL, &type, (LPBYTE) &LongDelay, &len );
		RegQueryValueEx( key, "ShortDelay", NULL, &type, (LPBYTE) &ShortDelay, &len );
		RegQueryValueEx( key, "DisplayCols", NULL, &type, (LPBYTE) &cols, &len );
		RegQueryValueEx( key, "DisplayRows", NULL, &type, (LPBYTE) &rows, &len );
		RegQueryValueEx( key, "CharWidth", NULL, &type, (LPBYTE) &cw, &len );
		RegQueryValueEx( key, "CharHeight", NULL, &type, (LPBYTE) &ch, &len );

		RegQueryValueEx( key, "BasePort", NULL, &type, (LPBYTE) &LPT_Data, &len );
		LPT_Ctrl = LPT_Data + 2;

		RegCloseKey( key );
	}

}


void LCD_Wait( int d )
{
	Sleep( d );
}

int LCD_Control( UCHAR b, int delay = 0 )
{
	DlPortWritePortUchar( LPT_Data, b);
	DlPortWritePortUchar( LPT_Ctrl, 0x02 );
	LCD_Wait( ShortDelay );
	DlPortWritePortUchar( LPT_Ctrl, 0x03 );

	LCD_Wait( delay );

	return 0;

}

int LCD_Data( UCHAR b, int delay = 0 )
{
	DlPortWritePortUchar( LPT_Data, b);
	DlPortWritePortUchar( LPT_Ctrl, 0x06 );
	LCD_Wait( ShortDelay );
	DlPortWritePortUchar( LPT_Ctrl, 0x05 );

	LCD_Wait( delay );

	return 0;

}

int LCD_Set_Pos( int x, int y )
{
	int d = x + (y % 2) * 0x40;

	if ((y % 4) >= 2)
		d += cols;

	if ( rows == 1 )
		d += (x % 8) * (0x40 - 8);       

	LCD_Control( 0x80 + d, LongDelay );

	return 0;
}

int LCD_Clear( void )
{
	LCD_Control( 0x1, LongDelay );    //Clear Display
	return 0;
}

char* mklogtime ( void )
{
	static char buf[256];
	sprintf( buf, "%i ", time(NULL) );
	return buf;
}

extern "C"
{

FILE* log;

//needed
int __stdcall lcdGetWidth(void)
{
	LOG( fprintf( log, "%slcdGetWidth()\n", mklogtime() ));
	return cols;
}

//needed
int __stdcall lcdGetHeight(void)
{
	LOG( fprintf( log, "%slcdGetHeight()\n", mklogtime() ));
	return rows;
}

int __stdcall lcdGetCharWidth(void)
{
	LOG( fprintf( log, "%slcdGetCharWidth()\n", mklogtime() ));
	return cw;
}

int __stdcall lcdGetCharHeight(void)
{
	LOG( fprintf( log, "%slcdGetCharHeight()\n", mklogtime() ));
	return ch;
}

//needed
int __stdcall lcdRequest(void)
{

#ifdef _DEBUG
	log = fopen( "c:\\lcd.log", "a" );
	if( log ) fprintf( log, "%slcdRequest()\n", mklogtime() );
#endif

	LCD_Config();

	LCD_Control( 0x38, LongDelay);   //Set 8Bit Mode (5x7 dots, 1/16 Duty (2 lines))
    LCD_Control( 0x38, LongDelay);   //Set 8Bit Mode ONE MORE TIME
    LCD_Control( 0x6,  ShortDelay);    //Entry Mode Set
    LCD_Control( 0xC,  ShortDelay);    //Display on, cursor off, blink off
    LCD_Set_Pos( 0, 0 );
    LCD_Clear();               //Clear Display

	return LCD_RESULT_NOERROR;
}

//needed
int __stdcall lcdFree(void)
{
#ifdef _DEBUG
	if( log ) fprintf( log, "%slcdFree()\n", mklogtime() );
	if( log ) fclose( log );
#endif
	return LCD_RESULT_NOERROR;
}

//needed
int __stdcall lcdClearDisplay(void)
{
	LOG( fprintf( log, "%slcdClearDisplay()\n", mklogtime() ));
    LCD_Clear();               //Clear Display
	return LCD_RESULT_NOERROR;
}

//needed
int __stdcall lcdSetWrapMode(int fHoriz, int fVert)
{
	LOG( fprintf( log, "%slcdSetWrapMode( %i, %i)\n", mklogtime(), fHoriz, fVert ));
	return LCD_RESULT_NOERROR;
}

extern int __stdcall lcdSetCustomChar(int iChar, const char *cBytes);

//needed
int __stdcall lcdSetCursor(int iCursorMode)
{
	LOG( fprintf( log, "%slcdSetCursor( %i )\n", mklogtime(), iCursorMode ));
	return LCD_RESULT_NOERROR;
}

//needed
int __stdcall lcdSetCursorPos(int iX, int iY)
{
	LOG( fprintf( log, "%slcdSetCursorPos( %i, %i)\n", mklogtime(), iX, iY ));

    LCD_Set_Pos( iX, iY );

	return LCD_RESULT_NOERROR;
}

extern int __stdcall lcdSendCharacter(unsigned char cChar);

//needed
int __stdcall lcdSendString(const char *pcString, unsigned int iLength)
{
	char s[512];
	strncpy( s, pcString, iLength );
	s[ iLength ] = 0;

	unsigned int i = 0;
	for( i = 0; i < iLength; i++ )
	{
		LCD_Data( (UCHAR) s[i], LongDelay );
		LOG(  fprintf( log, "%ssending( %i)\n", mklogtime(), s[i] ));
	}

	LOG( fprintf( log, "%slcdSendString( '%s', %i)\n", mklogtime(), s, iLength ));
	return LCD_RESULT_NOERROR;
}

} // extern "C"